// Copyright 1999 Omni Development, Inc.  All rights reserved.
//
// This software may only be used and reproduced according to the
// terms in the file OmniSourceLicense.html, which should be
// distributed with this project and can also be found at
// http://www.omnigroup.com/DeveloperResources/OmniSourceLicense.html.


#define RCS_ID(rcsIdString) \
        static void *rcs_id = rcsIdString; \
        static void *__rcs_id_hack() { __rcs_id_hack(); return rcs_id; }

RCS_ID("$Header: /Network/Source/CVS/OmniGroup/Bundles/Preferences/EnvironmentVariables/EnvironmentVariables.m,v 1.2 1999/10/12 02:47:41 andrew Exp $")


#import "EnvironmentVariables.h"


#define NEW_KEY_NAME	@"NewVariable"


@implementation EnvironmentVariables

- (id)init;
{
    if (!(self = [super init]))
        return nil;


    if (![NSBundle loadNibNamed: @"EnvironmentVariables" owner:self]) {
        NSRunAlertPanel(@"Error", @"Unable to load EnvironmentVariables.nib.", @"OK", nil, nil);
        [self release];
        return nil;
    }

    keys = [[NSMutableArray alloc] initWithCapacity:20];

    [window setFrameAutosaveName:@"Environment Variables"];

    [tableView setDataSource:self];
    [tableView setDelegate:self];

    return self;
}

- (void)dealloc;
{
    [keys release];
    [variables release];

    [super dealloc];
}

- (void)willSelect:(id)sender;
{
    static BOOL hasSelectedBefore  = NO;
    
    [super willSelect:sender];
    if (!hasSelectedBefore) {
        hasSelectedBefore = YES;
        [self revert:nil];
    }
}

- (void)setModified:(BOOL)flag;
{
    [window setDocumentEdited:flag];
    [revertButtonCell setEnabled:flag];
    [saveButtonCell setEnabled:flag];
}

- (void)updateKeysArray;
{
    [keys removeAllObjects];
    [keys addObjectsFromArray:[[variables allKeys] sortedArrayUsingSelector:@selector(compare:)]];
}

- (IBAction)addVariable:(id)sender;
{
    NSString *keyName;
    NSString *value;
    int rowIndex;
    int newKeyNumber;

    if (![window makeFirstResponder:window]) {
        NSBeep();
        return;
    }
    
    keyName = NEW_KEY_NAME;
    value = [variables objectForKey:keyName];

    newKeyNumber = 1;
    while (value) {
        keyName = [NSString stringWithFormat:@"%@%d", NEW_KEY_NAME, newKeyNumber++];
        value = [variables objectForKey:keyName];
    }
    [variables setObject:@"" forKey:keyName];
    [self updateKeysArray];

    [tableView reloadData];

    rowIndex = [keys indexOfObject:keyName];
    [tableView selectRow:rowIndex byExtendingSelection:NO];
    [tableView editColumn:0 row:rowIndex withEvent:nil select:YES];

    [self setModified:YES];
}

- (IBAction)deleteVariable:(id)sender;
{
    int rowIndex;
    NSString *key;

    if (![window makeFirstResponder:window]) {
        NSBeep();
        return;
    }
    
    rowIndex = [tableView selectedRow];
    if (rowIndex == -1)
        return;
    
    key = [keys objectAtIndex:rowIndex];
    [variables removeObjectForKey:key];
    [keys removeObject:key];

    [tableView reloadData];

    [self setModified:YES];
}

- (IBAction)save:(id)sender;
{
    if (![window makeFirstResponder:window]) {
        NSBeep();
        return;
    }

    if ([variables writeToFile:[@"~/.OpenStep/environment" stringByStandardizingPath] atomically:YES])
        [self setModified:NO];
    else
        NSRunAlertPanel(@"Error", @"Unable to save environment file.", @"OK", nil, nil);
}

- (IBAction)revert:(id)sender;
{
    int rowIndex;
    NSString *selectedKey  = nil;
    
    if (![window makeFirstResponder:window]) {
        NSBeep();
        return;
    }

    rowIndex = [tableView selectedRow];
    if (rowIndex != -1)
        selectedKey = [[[keys objectAtIndex:rowIndex] retain] autorelease];
    
    [variables release];
    variables = [[NSMutableDictionary alloc] initWithContentsOfFile:[@"~/.OpenStep/environment" stringByStandardizingPath]];
    if (!variables) {
        NSLog (@"Unable to locate environment file.");
        variables = [[NSMutableDictionary alloc] initWithCapacity:5];
        [keys removeAllObjects];
    }
    
    [self updateKeysArray];
    [tableView reloadData];

    if (selectedKey) {
        rowIndex = [keys indexOfObject:selectedKey];
        if (rowIndex != NSNotFound)
            [tableView selectRow:rowIndex byExtendingSelection:NO];
        else
            [tableView selectRow:-1 byExtendingSelection:NO];
    }

    [self setModified:NO];
}

// NSTableView data source

- (int)numberOfRowsInTableView:(NSTableView *)aTableView
{
    return [keys count];
}

- (id)tableView:(NSTableView *)aTableView objectValueForTableColumn:(NSTableColumn *)aTableColumn row:(int)rowIndex;
{
    NSString *key;
    NSString *identifier;

    key = [keys objectAtIndex:rowIndex];
    identifier = [aTableColumn identifier];
    
    if ([identifier isEqualToString:@"key"])
        return key;
    else if ([identifier isEqualToString:@"value"])
        return [variables objectForKey:key];

    return nil;
}

- (void)tableView:(NSTableView *)aTableView setObjectValue:(id)anObject forTableColumn:(NSTableColumn *)aTableColumn row:(int)rowIndex ;
{
    NSString *key;
    NSString *identifier;
    NSString *oldValue;

    key = [keys objectAtIndex:rowIndex];
    identifier = [aTableColumn identifier];

    if ([identifier isEqualToString:@"key"]) {
        if ([key isEqualToString:anObject])
            return;
        oldValue = [variables objectForKey:key];
        [variables setObject:oldValue forKey:anObject];
        [keys replaceObjectAtIndex:rowIndex withObject:anObject];

    } else if ([identifier isEqualToString:@"value"]) {
        oldValue = [variables objectForKey:key];
        if ([oldValue isEqualToString:anObject])
            return;
        [variables setObject:anObject forKey:key];
    }

    [self setModified:YES];
}

@end
